from abc import ABCMeta, abstractmethod
from dataclasses import dataclass


class Event(metaclass = ABCMeta):
	pass


@dataclass
class UpdateEvent(Event):
	payload_data: dict  # object at "payload" -> "data" in the message JSON


@dataclass
class DeleteEvent(Event):
	payload: str  # update name


class EventStream(metaclass = ABCMeta):
	@abstractmethod
	async def get_event(self) -> Event:
		raise NotImplementedError()


@dataclass
class NotifyInfo:
	recipient: str
	thread_id: str
	thread_title: str
	update_id: str
	author: str
	update_body: str


class Notifier(metaclass = ABCMeta):
	@abstractmethod
	def notify(self, info: NotifyInfo) -> None:
		raise NotImplementedError()


class ThreadTitleProvider(metaclass = ABCMeta):
	@abstractmethod
	def title_for(self, thread_id: str) -> str:
		raise NotImplementedError()


class UpdateChecker(metaclass = ABCMeta):
	@abstractmethod
	def exists(self, thread_id: str, update_id: str) -> bool:
		raise NotImplementedError()
